import React from "react";
import Step from "./Step.js";
export interface StepperProps extends React.HTMLAttributes<HTMLOListElement> {
    /**
     * `<Stepper.Step />` elements.
     */
    children: React.ReactNode;
    /**
     * The direction the component grows.
     * @default "vertical"
     */
    orientation?: "horizontal" | "vertical";
    /**
     * Current active step.
     *
     * Stepper index starts at 1, not 0.
     */
    activeStep: number;
    /**
     * Callback for next `activeStep`.
     *
     * Stepper index starts at 1, not 0.
     */
    onStepChange?: (step: number) => void;
    /**
     * Makes stepper non-interactive if false.
     * @default true
     */
    interactive?: boolean;
}
interface StepperComponent extends React.ForwardRefExoticComponent<StepperProps & React.RefAttributes<HTMLOListElement>> {
    /**
     * @see 🏷️ {@link StepperStepProps}
     * @see [🤖 OverridableComponent](https://aksel.nav.no/grunnleggende/kode/overridablecomponent) support
     */
    Step: typeof Step;
}
/**
 * A component that displays a stepper with clickable steps.
 *
 * @see [📝 Documentation](https://aksel.nav.no/komponenter/core/stepper)
 * @see 🏷️ {@link StepperProps}
 *
 * @example
 * ```jsx
 * <>
 *   <Heading size="medium" spacing level="2" id="stepper-heading">
 *     Søknadssteg
 *   </Heading>
 *   <Stepper
 *     aria-labelledby="stepper-heading"
 *     activeStep={activeStep}
 *     onStepChange={setActiveStep}
 *   >
 *     <Stepper.Step href="#">Start søknad</Stepper.Step>
 *     <Stepper.Step href="#">Saksopplysninger</Stepper.Step>
 *     <Stepper.Step href="#">Vedlegg</Stepper.Step>
 *   </Stepper>
 * </>
 * ```
 */
export declare const Stepper: StepperComponent;
export default Stepper;
